//*****************************************************************************
//                               NbkNgSpice.cpp                               *
//                              ----------------                              *
//  Started     : 08/05/2004                                                  *
//  Last Update : 05/09/2006                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "ngspice/NbkNgSpice.hpp"

//*****************************************************************************
// Constructor.
//
// Arguments:
//   poParent - The parent window
//   oWinID   - The display object ID
//   roPosn   - The notebook position
//   roSize   - The notebook size

NbkNgSpice::NbkNgSpice( wxWindow * poParent, wxWindowID oWinID )
                   : NbkSimrBase( poParent, oWinID ), m_oDlgOptions( poParent )
{
  // Set the simulator engine type specifier
  bSetSimrType( Simulation::eSIMR_NGSPICE );

  // Create the various display objects
  m_poPnlNgSpiceDC = new PnlNgSpiceDC( this );
  m_poPnlNgSpiceAC = new PnlNgSpiceAC( this );
  m_poPnlNgSpiceTR = new PnlNgSpiceTR( this );

  // Add the display objects to the note book
  AddPage( m_poPnlNgSpiceDC, wxT( "     DC     " ) );
  AddPage( m_poPnlNgSpiceAC, wxT( "     AC     " ) );
  AddPage( m_poPnlNgSpiceTR, wxT( " Transient  " ) );

  SetSelection( 0 ); // Specify the page to be displayed
}

//*****************************************************************************
// Default constructor.
// (Used for two stage creation ie. must call Create( ) method.

NbkNgSpice::NbkNgSpice( void ) : NbkSimrBase( ), m_oDlgOptions( this )
{
  // Set the simulator engine type specifier
  bSetSimrType( Simulation::eSIMR_NGSPICE );
}

//*****************************************************************************
// Destructor.

NbkNgSpice::~NbkNgSpice( )
{
}

//*****************************************************************************
// Clear the object attributes.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bClear( void )
{
  bool  bRtn=TRUE;

  if( ! m_oDlgOptions    .bClear( ) ) bRtn = FALSE;

  if( ! m_poPnlNgSpiceDC->bClear( ) ) bRtn = FALSE;
  if( ! m_poPnlNgSpiceAC->bClear( ) ) bRtn = FALSE;
  if( ! m_poPnlNgSpiceTR->bClear( ) ) bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Load information from a simulation object.
//
// Argument List:
//   roSimultn - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bLoad( Simulation & roSim )
{
  bool  bRtn=TRUE;

  if( m_oDlgOptions.bSetCmd( roSim.rosGetOPTIONS( ) ) ) bRtn = FALSE;

  if( ! m_poPnlNgSpiceDC->bLoad( roSim ) )              bRtn = FALSE;
  if( ! m_poPnlNgSpiceAC->bLoad( roSim ) )              bRtn = FALSE;
  if( ! m_poPnlNgSpiceTR->bLoad( roSim ) )              bRtn = FALSE;

  if( ! bSetPage( roSim.eGetAnaType( ) ) )              bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Reload information from a simulation object.
//
// Argument List:
//   roSimultn - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bReload( Simulation & roSim )
{
  bool  bRtn=TRUE;

  if( m_oDlgOptions.bSetCmd( roSim.rosGetOPTIONS( ) ) ) bRtn = FALSE;

  if( ! m_poPnlNgSpiceDC->bReload( roSim ) )            bRtn = FALSE;
  if( ! m_poPnlNgSpiceAC->bReload( roSim ) )            bRtn = FALSE;
  if( ! m_poPnlNgSpiceTR->bReload( roSim ) )            bRtn = FALSE;

  if( ! bSetPage( roSim.eGetAnaType( ) ) )              bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Save information to a simulation object.
//
// Argument List:
//   roSimultn - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bSave( Simulation & roSim )
{
  roSim.bSetOPTIONS( m_oDlgOptions.rosGetCmd( ) );

  switch( GetSelection( ) )
  {
    case 0: // DC analysis
      if( ! m_poPnlNgSpiceDC->bSave( roSim ) ) return( FALSE );
      break;
    case 1: // AC analysis
      if( ! m_poPnlNgSpiceAC->bSave( roSim ) ) return( FALSE );
      break;
    case 2: // TR analysis
      if( ! m_poPnlNgSpiceTR->bSave( roSim ) ) return( FALSE );
      break;
    default:                                   return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Set the page to be displayed.
//
// Argument List:
//   eType - The enumerated analysis type specifier
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bSetPage( Simulation::eAnaType eType )
{
  int  iPage;

  switch( eType )
  {
    case Simulation::eANA_OP:
    case Simulation::eANA_DC: iPage = 0; break;
    case Simulation::eANA_AC: iPage = 1; break;
    case Simulation::eANA_TR: iPage = 2; break;
    default:                  return( FALSE );
  }

  SetSelection( iPage ); // Specify the page to be displayed

  return( TRUE );
}

//*****************************************************************************
// Set the page to be displayed.
//
// Argument List:
//   psType - The two letter analysis type specifier (case ignored)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  NbkNgSpice::bSetPage( const wxChar * psType )
{
  Simulation::eAnaType  eType=Simulation::eANA_NONE;
  wxString  osType;

  osType = wxString( psType ).Upper( );

  if( osType.Length( ) == 2 )
  {
         if( osType == wxT("OP") ) eType = Simulation::eANA_DC;
    else if( osType == wxT("DC") ) eType = Simulation::eANA_DC;
    else if( osType == wxT("AC") ) eType = Simulation::eANA_AC;
    else if( osType == wxT("TR") ) eType = Simulation::eANA_TR;
  }

  return( bSetPage( eType ) );
}

//*****************************************************************************
// Get the two letter page specifier.
//
// Return Values:
//   Success - The two letter analysis type specifier (lower case)
//   Failure - An empty string

const wxString & NbkNgSpice::rosGetPage( void )
{
  static  wxString  osPage;

  switch( eGetPage( ) )
  {
    case Simulation::eANA_DC: osPage = wxT("dc"); break;
    case Simulation::eANA_AC: osPage = wxT("ac"); break;
    case Simulation::eANA_TR: osPage = wxT("tr"); break;
    default:                  osPage.Empty( );
  }

  return( osPage );
}

//*****************************************************************************
// Get the enumerated page specifier.
//
// Return Values:
//   Success - The enumerated analysis specifier
//   Failure - Simulation::eANA_NONE

Simulation::eAnaType  NbkNgSpice::eGetPage( void )
{
  Simulation::eAnaType  ePage;

  switch( GetSelection( ) )
  {
    case 0:  ePage = Simulation::eANA_DC; break;
    case 1:  ePage = Simulation::eANA_AC; break;
    case 2:  ePage = Simulation::eANA_TR; break;
    default: ePage = Simulation::eANA_NONE;
  }

  return( ePage );
}

//*****************************************************************************
// Display the OPTIONS line setup dialog.
//
// Return Values:
//   Success - TRUE
//   Failure - FALSE

bool  NbkNgSpice::bShowDlgOptions( void )
{
  PnlAnaBase * poPnlAna;
  double  df1;

  // Set the temperature in the OPTIONS dialog
  poPnlAna = (PnlAnaBase *) GetPage( GetSelection( ) );
  if( poPnlAna->bIsCreatedTemp( ) )
  {
    df1 = poPnlAna->m_oPnlTemp.dfGetValue( );
    m_oDlgOptions.bSetValue( DlgNgSpiceOPT::ID_PNL_TEMP, (float) df1 );
  }
  else poPnlAna = NULL;

  // Display the OPTIONS dialog
  m_oDlgOptions.CenterOnParent( );
  m_oDlgOptions.ShowModal( );

  // Set the temperature in the analysis panel
  if( poPnlAna != NULL )
  {
    df1 = m_oDlgOptions.dfGetValue( DlgNgSpiceOPT::ID_PNL_TEMP );
    poPnlAna->m_oPnlTemp.bSetValue( df1 );
  }
}

//*****************************************************************************
